function light_level_to_time_of_day(light_level)
    if light_level <= 0.14 then
        return 0.5
    elseif light_level > 0.99 then
        return 0.8
    end
    return 0.2 * light_level + 0.55
end


script.on_event(defines.events.on_tick, function(event)
    local light = get_available_light()
    global.current_light = light
    local time_of_day = light_level_to_time_of_day(light)
    game.surfaces["nauvis"].daytime = time_of_day
end)

light_trends = {brightening = {min_length = 60 * 60 * 2, max_length = 60 * 60 * 10, odds = 50, intensity =  0.4},
                clear =       {min_length = 60 * 60 * 1, max_length = 60 * 60 * 3,  odds = 65, intensity =  0.1},
                gloomy =      {min_length = 60 * 60 * 3, max_length = 60 * 60 * 12, odds = 80, intensity = -0.2},
                darkening =   {min_length = 60 * 60 * 2, max_length = 60 * 60 * 7,  odds = 60, intensity = -0.5}
                }

function get_available_light()
    if global.light_queue then global.light_queue = nil end
    if not global.light_trend then begin_new_light_trend("clear") end
    
    if global.trend_expires < game.tick then
        begin_new_light_trend(choose_light_trend(global.light_trend))
    end

    return math.min(1, math.max(0.14, global.current_light + global.trend_delta))
end

function begin_new_light_trend(trend)
    local data = light_trends[trend]
    local time_length = math.random(data.min_length, data.max_length)
    global.trend_expires = game.tick + time_length
    global.trend_delta = data.intensity / time_length
    global.light_trend = trend
    global.current_light = 1 - game.surfaces["nauvis"].darkness
    Logger.log("Beginning trend: " .. trend .. ", Expires in " .. (time_length / 60) .. "s, Delta: " .. global.trend_delta)
end

function choose_light_trend(excluded_trend)
    local best_roll = 0
    local best_trend = nil
    for trend, data in pairs(light_trends) do
        if trend ~= excluded_trend then
            local roll = math.random(data.odds)
            if roll > best_roll then
                best_roll = roll
                best_trend = trend
            end
        end
    end
    
    return best_trend
end

Logger = {prefix='gloom', name = 'main', log_buffer = {}, last_write_tick = 0, last_write_size = 0, ever_written = false, debug = false}

function Logger.log(str)
    local run_time_s = 0
    local run_time_minutes = 0
    local run_time_hours = 0
    if _G["game"] then
        run_time_s = math.floor(game.tick/60)
        run_time_minutes = math.floor(run_time_s/60)
        run_time_hours = math.floor(run_time_minutes/60)
    end
    Logger.log_buffer[#Logger.log_buffer + 1] = string.format("%02d:%02d:%02d: %s\r\n", run_time_hours, run_time_minutes % 60, run_time_s % 60, str)
    Logger.checkOutput()
end

function Logger.checkOutput()
    if _G["game"] then
        if Logger.last_write_size ~= #Logger.log_buffer and (debug or (game.tick - Logger.last_write_tick) > 3600) then
            Logger.dump()
        end
    end
end

function Logger.dump()
    if _G["game"] then
        Logger.last_write_tick = game.tick
        Logger.last_write_size = #Logger.log_buffer
        local file_name = "logs/" .. Logger.prefix .. "/" .. Logger.name .. ".log"
        game.write_file(file_name, table.concat(Logger.log_buffer), Logger.ever_written)
        Logger.log_buffer = {}
        Logger.ever_written = true
        return true
    end
    return false
end